@@ -1,7 +1,7 @@
 /*
  * xgamelist.c -- Game list window, part of X front end for XBoard
  *
- * Copyright 1995, 2009, 2010, 2011 Free Software Foundation, Inc.
+ * Copyright 1995, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
  * ------------------------------------------------------------------------
  *
  * GNU XBoard is free software: you can redistribute it and/or modify
@@ -60,6 +60,7 @@ extern char *getenv();
 #include <X11/Xaw3d/Text.h>
 #include <X11/Xaw3d/AsciiText.h>
 #include <X11/Xaw3d/Viewport.h>
+#include <X11/Xaw3d/Scrollbar.h>
 #else
 #include <X11/Xaw/Dialog.h>
 #include <X11/Xaw/Form.h>
@@ -73,6 +74,7 @@ extern char *getenv();
 #include <X11/Xaw/Text.h>
 #include <X11/Xaw/AsciiText.h>
 #include <X11/Xaw/Viewport.h>
+#include <X11/Xaw/Scrollbar.h>
 #endif
 
 #include "common.h"
@@ -96,6 +98,8 @@ void SetFocus P((Widget w, XtPointer data, XEvent *event, Boolean *b));
 static Widget filterText;
 static char filterString[MSG_SIZ];
 static int listLength, wins, losses, draws, page;
+static char *list[1003];
+int listEnd;
 
 char gameListTranslations[] =
   "<Btn1Up>(2): LoadSelectedProc(0) \n \
@@ -124,11 +128,66 @@ typedef struct {
 } GameListClosure;
 static GameListClosure *glc = NULL;
 
+int
+ReadScroll (Widget listwidg, float *top, float *bottom)
+{   // retreives fractions of top and bottom of thumb
+    Arg args[16];
+    Widget w = XtParent(listwidg); // viewport
+    Widget v = XtNameToWidget(w, "vertical");
+    int j=0;
+    float h;
+    if(!v) return FALSE; // no scroll bar
+    XtSetArg(args[j], XtNshown, &h); j++;
+    XtSetArg(args[j], XtNtopOfThumb, top); j++;
+    XtGetValues(v, args, j);
+    *bottom = *top + h;
+    return TRUE;
+}
+
+void
+SetScroll (Widget listwidg, float f)
+{   // sets top of thumb to given fraction
+    static char *params[3] = { "", "Continuous", "Proportional" };
+    static XEvent event;
+    Widget w = XtParent(listwidg); // viewport
+    Widget v = XtNameToWidget(w, "vertical");
+    if(!v) return; // no scroll bar
+    XtCallActionProc(v, "StartScroll", &event, params+1, 1);
+    XawScrollbarSetThumb(v, f, -1.0);
+    XtCallActionProc(v, "NotifyThumb", &event, params, 0);
+//    XtCallActionProc(v, "NotifyScroll", &event, params+2, 1);
+    XtCallActionProc(v, "EndScroll", &event, params, 0);
+}
+
+void
+HighlightWithScroll (Widget listwidg, int sel, int max)
+{
+    float top, bottom, f, g;
+    XawListHighlight(listwidg, sel);
+    if(!ReadScroll(listwidg, &top, &bottom)) return; // no scroll bar
+    bottom = bottom*max - 1.;
+    f = g = top;
+    top *= max;
+    if(sel > (top + 3*bottom)/4) f = (sel - 0.75*(bottom-top))/max; else
+    if(sel < (3*top + bottom)/4) f = (sel - 0.25*(bottom-top))/max;
+    if(f < 0.) f = 0.; if(f + 1./max > 1.) f = 1. - 1./max;
+    if(f != g) SetScroll(listwidg, f);
+}
+
+void
+GameListHighlight (int index)
+{
+    Widget listwidg;
+    int i=0; char **st;
+    if (glc == NULL || !glc->up) return;
+    listwidg = XtNameToWidget(glc->shell, "*form.viewport.list");
+    st = list;
+    while(*st && atoi(*st)<index) st++,i++;
+    HighlightWithScroll(listwidg, i, listLength);
+}
+
 Widget
-GameListCreate(name, callback, client_data)
-     char *name;
-     XtCallbackProc callback;
-     XtPointer client_data;
+GameListCreate (char *name, XtCallbackProc callback, XtPointer client_data)
 {
     Arg args[16];
     Widget shell, form, viewport, listwidg, layout, label;
@@ -144,15 +203,9 @@ GameListCreate(name, callback, client_data)
     j = 0;
     XtSetArg(args[j], XtNresizable, True);  j++;
     XtSetArg(args[j], XtNallowShellResize, True);  j++;
-#if TOPLEVEL
     shell = gameListShell =
-      XtCreatePopupShell(name, topLevelShellWidgetClass,
+      XtCreatePopupShell(name, appData.topLevel ? topLevelShellWidgetClass : transientShellWidgetClass,
 			 shellWidget, args, j);
-#else
-    shell = gameListShell =
-      XtCreatePopupShell(name, transientShellWidgetClass,
-			 shellWidget, args, j);
-#endif
     layout =
       XtCreateManagedWidget(layoutName, formWidgetClass, shell,
 			    layoutArgs, XtNumber(layoutArgs));
@@ -330,16 +383,14 @@ extern int soughtCounts[];
 extern Board soughtBoard;
 
 static int
-GameListPrepare(int byPos)
+GameListPrepare (int byPos)
 {   // [HGM] filter: put in separate routine, to make callable from call-back
     int nstrings;
     ListGame *lg;
     char **st, *line;
-struct {
-    long sec;  /* Assuming this is >= 32 bits */
-    int ms;    /* Assuming this is >= 16 bits */
-} t,t2; GetTimeMark(&t);
+    TimeMark t, t2;
 
+    GetTimeMark(&t);
     if(st = glc->strings) while(*st) free(*st++);
     nstrings = ((ListGame *) gameList.tailPred)->number;
     glc->strings = (char **) malloc((nstrings + 1) * sizeof(char *));
@@ -365,17 +416,14 @@ struct {
 	lg->position = pos;
 	lg = (ListGame *) lg->node.succ;
      }
-GetTimeMark(&t2);printf("GameListPrepare %d msec\n", SubtractTimeMarks(&t2,&t));
+GetTimeMark(&t2);printf("GameListPrepare %ld msec\n", SubtractTimeMarks(&t2,&t));
      DisplayTitle("XBoard");
     *st = NULL;
     return listLength;
 }
 
-static char *list[1003];
-int listEnd;
-
 static void
-GameListReplace(int page)
+GameListReplace (int page)
 {
   // filter: put in separate routine, to make callable from call-back
   Widget listwidg;
@@ -393,16 +441,14 @@ GameListReplace(int page)
   XtSetArg(arg, XtNlist, listLength ? list : dummyList); // empty list displays message
   XawListChange(listwidg, list, 0, 0, True);
   XtSetValues(listwidg, &arg, 1);
-  XawListHighlight(listwidg, 0);
-  snprintf(buf, MSG_SIZ, "%s- %d/%d games (%d-%d-%d)", glc->filename, listLength, ((ListGame *) gameList.tailPred)->number, wins, losses, draws);
+  HighlightWithScroll(listwidg, 0, listLength);
+  snprintf(buf, MSG_SIZ, _("%s - %d/%d games (%d-%d-%d)"), glc->filename, listLength, ((ListGame *) gameList.tailPred)->number, wins, losses, draws);
   XtSetArg(arg, XtNtitle, buf);
   XtSetValues(glc->shell, &arg, 1);
 }
 
 void
-GameListCallback(w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
+GameListCallback (Widget w, XtPointer client_data, XtPointer call_data)
 {
     String name;
     Arg args[16];
@@ -438,14 +484,14 @@ GameListCallback(w, client_data, call_data)
 	    DisplayError(_("Can't go forward any further"), 0);
 	    return;
 	}
-	XawListHighlight(listwidg, index);
+	HighlightWithScroll(listwidg, index, listLength);
     } else if (strcmp(name, _("prev")) == 0) {
 	index = rs->list_index - 1;
 	if (index < 0) {
 	    DisplayError(_("Can't back up any further"), 0);
 	    return;
 	}
-	XawListHighlight(listwidg, index);
+	HighlightWithScroll(listwidg, index, listLength);
     } else if (strcmp(name, _("apply")) == 0 ||
                strcmp(name, _("find position")) == 0) {
         String text;
@@ -453,7 +499,7 @@ GameListCallback(w, client_data, call_data)
         XtSetArg(args[j], XtNstring, &text);  j++;
 	XtGetValues(filterText, args, j);
         safeStrCpy(filterString, text, sizeof(filterString)/sizeof(filterString[0]));
-	XawListHighlight(listwidg, 0);
+	HighlightWithScroll(listwidg, 0, listLength);
         GameListPrepare(strcmp(name, _("find position")) == 0); GameListReplace(0);
         return;
     }
@@ -470,9 +516,7 @@ GameListCallback(w, client_data, call_data)
 }
 
 void
-GameListPopUp(fp, filename)
-     FILE *fp;
-     char *filename;
+GameListPopUp (FILE *fp, char *filename)
 {
     Arg args[16];
     int j;
@@ -481,6 +525,8 @@ GameListPopUp(fp, filename)
     if (glc == NULL) {
 	glc = (GameListClosure *) calloc(1, sizeof(GameListClosure));
 	glc->x = glc->y = -1;
+	glc->filename = NULL;
+	glc->shell = NULL;
     }
 
     GameListPrepare(False); // [HGM] filter: code put in separate routine
@@ -490,7 +536,6 @@ GameListPopUp(fp, filename)
     if (glc->filename != NULL) free(glc->filename);
     glc->filename = StrSave(filename);
 
-
     if (glc->shell == NULL) {
 	glc->shell = GameListCreate(filename, GameListCallback, glc);
     } else {
@@ -511,7 +556,7 @@ GameListPopUp(fp, filename)
 }
 
 void
-GameListDestroy()
+GameListDestroy ()
 {
     if (glc == NULL) return;
     GameListPopDown();
@@ -528,11 +573,7 @@ GameListDestroy()
 }
 
 void
-ShowGameListProc(w, event, prms, nprms)
-     Widget w;
-     XEvent *event;
-     String *prms;
-     Cardinal *nprms;
+ShowGameListProc (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
     Arg args[16];
     int j;
@@ -555,11 +596,7 @@ ShowGameListProc(w, event, prms, nprms)
 }
 
 void
-LoadSelectedProc(w, event, prms, nprms)
-     Widget w;
-     XEvent *event;
-     String *prms;
-     Cardinal *nprms;
+LoadSelectedProc (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
     Widget listwidg;
     XawListReturnStruct *rs;
@@ -589,7 +626,7 @@ LoadSelectedProc(w, event, prms, nprms)
 	if(direction == -2) index = 0;
 	if(direction == 2) index = listEnd-1;
 	if(index < 0 || index >= listEnd) return;
-	XawListHighlight(listwidg, index);
+	HighlightWithScroll(listwidg, index, listLength);
 	if(!doLoad) return;
     }
     index = atoi(list[index])-1; // [HGM] filter: read true index from sequence nr of line
@@ -603,11 +640,7 @@ LoadSelectedProc(w, event, prms, nprms)
 }
 
 void
-SetFilterProc(w, event, prms, nprms)
-     Widget w;
-     XEvent *event;
-     String *prms;
-     Cardinal *nprms;
+SetFilterProc (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
 	Arg args[16];
         String name;
@@ -618,7 +651,7 @@ SetFilterProc(w, event, prms, nprms)
         safeStrCpy(filterString, name, sizeof(filterString)/sizeof(filterString[0]));
         GameListPrepare(False); GameListReplace(0);
 	list = XtNameToWidget(glc->shell, "*form.viewport.list");
-	XawListHighlight(list, 0);
+	HighlightWithScroll(list, 0, listLength);
         j = 0;
 	XtSetArg(args[j], XtNdisplayCaret, False); j++;
 	XtSetValues(filterText, args, j);
@@ -626,7 +659,7 @@ SetFilterProc(w, event, prms, nprms)
 }
 
 void
-GameListPopDown()
+GameListPopDown ()
 {
     Arg args[16];
     int j;
@@ -651,26 +684,14 @@ GameListPopDown()
 		args, j);
 }
 
-void
-GameListHighlight(index)
-     int index;
-{
-    Widget listwidg;
-    int i=0; char **st;
-    if (glc == NULL || !glc->up) return;
-    listwidg = XtNameToWidget(glc->shell, "*form.viewport.list");
-    st = list;
-    while(*st && atoi(*st)<index) st++,i++;
-    XawListHighlight(listwidg, i);
-}
-
 Boolean
-GameListIsUp()
+GameListIsUp ()
 {
     return glc && glc->up;
 }
 
-int SaveGameListAsText(FILE *f)
+int
+SaveGameListAsText (FILE *f)
 {
     ListGame * lg = (ListGame *) gameList.head;
     int nItem;
@@ -706,32 +727,36 @@ Widget gameListOptShell, listwidg;
 char *strings[20];
 int stringPtr;
 
-void GLT_ClearList()
+void
+GLT_ClearList ()
 {
     strings[0] = NULL;
     stringPtr = 0;
 }
 
-void GLT_AddToList(char *name)
+void
+GLT_AddToList (char *name)
 {
     strings[stringPtr++] = name;
     strings[stringPtr] = NULL;
 }
 
-Boolean GLT_GetFromList(int index, char *name)
+Boolean
+GLT_GetFromList (int index, char *name)
 {
   safeStrCpy(name, strings[index], MSG_SIZ);
   return TRUE;
 }
 
-void GLT_DeSelectList()
+void
+GLT_DeSelectList ()
 {
     XawListChange(listwidg, strings, 0, 0, True);
     XawListHighlight(listwidg, 0);
 }
 
 void
-GameListOptionsPopDown()
+GameListOptionsPopDown ()
 {
   if (gameListOptShell == NULL) return;
 
@@ -742,9 +767,7 @@ GameListOptionsPopDown()
 }
 
 void
-GameListOptionsCallback(w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
+GameListOptionsCallback (Widget w, XtPointer client_data, XtPointer call_data)
 {
     String name;
     Arg args[16];
@@ -795,7 +818,7 @@ GameListOptionsCallback(w, client_data, call_data)
 }
 
 Widget
-GameListOptionsCreate()
+GameListOptionsCreate ()
 {
     Arg args[16];
     Widget shell, form, viewport, layout;
@@ -896,7 +919,7 @@ GameListOptionsCreate()
 }
 
 void
-GameListOptionsPopUp(Widget w, XEvent *event, String *prms, Cardinal *nprms)
+GameListOptionsPopUp (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
   if (gameListOptShell == NULL)
     gameListOptShell = GameListOptionsCreate();
