@@ -250,7 +250,7 @@ CreateComboPopup (Widget parent, Option *option, int n)
 
 extern WindowPlacement wpComment, wpTags, wpMoveHistory;
 char *trialSound;
-static int oldCores, oldPonder;
+static int oldCores, oldPonder, oldShow, oldBlind;
 int MakeColors P((void));
 void CreateGCs P((int redo));
 void CreateAnyPieces P((void));
@@ -410,6 +410,8 @@ GeneralOptionsOK (int n)
 	int newPonder = appData.ponderNextMove;
 	appData.ponderNextMove = oldPonder;
 	PonderNextMoveEvent(newPonder);
+	if(!appData.highlightLastMove) ClearHighlights(), ClearPremoveHighlights();
+	if(oldShow != appData.showCoords || oldBlind != appData.blindfold) DrawPosition(TRUE, NULL);
 	return 1;
 }
 
@@ -631,7 +633,7 @@ Option loadOptions[] = {
 { 0, 0, 5000, NULL, (void*) &appData.eloThreshold2, "", NULL, Spin, N_("Elo of weakest player at least:") },
 { 0, 0, 5000, NULL, (void*) &appData.dateThreshold, "", NULL, Spin, N_("No games before year:") },
 { 0, 1, 50, NULL, (void*) &appData.stretch, "", NULL, Spin, N_("Minimum nr consecutive positions:") },
-{ 1, 0, 180, NULL, (void*) &searchMode, (char*) modeNames, modeValues, ComboBox, N_("Seach mode:") },
+{ 1, 0, 180, NULL, (void*) &searchMode, (char*) modeNames, modeValues, ComboBox, N_("Search mode:") },
 { 0, 0, 0, NULL, (void*) &appData.ignoreColors, "", NULL, CheckBox, N_("Also match reversed colors") },
 { 0, 0, 0, NULL, (void*) &appData.findMirror, "", NULL, CheckBox, N_("Also match left-right flipped position") },
 { 0,  0, 0, NULL, (void*) &LoadOptionsOK, "", NULL, EndMark , "" }
@@ -1050,7 +1052,9 @@ GenericPopUp (Option *option, char *title, int dlgNr)
      i = 0;
     XtSetArg(args[i], XtNresizable, True); i++;
     popup = shells[dlgNr] =
-      XtCreatePopupShell(title, transientShellWidgetClass,
+      XtCreatePopupShell(title, appData.topLevel && (dlgNr == 7 || dlgNr == 2 || dlgNr == 1 || dlgNr == 4) ?
+					topLevelShellWidgetClass :
+					transientShellWidgetClass,
 			 shellWidget, args, i);
 
     layout =
@@ -1427,6 +1431,7 @@ void
 OptionsProc (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
    oldPonder = appData.ponderNextMove;
+   oldShow = appData.showCoords; oldBlind = appData.blindfold;
    GenericPopUp(generalOptions, _("General Options"), 0);
 }
 
@@ -1759,7 +1764,7 @@ LoadEngineProc (Widget w, XEvent *event, String *prms, Cardinal *nprms)
    isUCI = storeVariant = v1 = useNick = False; addToList = hasBook = True; // defaults
    if(engineChoice) free(engineChoice); engineChoice = strdup(engineNr[0]);
    if(engineLine)   free(engineLine);   engineLine = strdup("");
-   if(engineDir)    free(engineDir);    engineDir = strdup("");
+   if(engineDir)    free(engineDir);    engineDir = strdup(".");
    if(nickName)     free(nickName);     nickName = strdup("");
    if(params)       free(params);       params = strdup("");
    NamesToList(firstChessProgramNames, engineList, engineMnemonic, "all");
